# ----------------------------------------------------------------------------------------------
# --------------- Measures for security assurance by Self Description ---------------
# ----------------------------------------------------------------------------------------------
fossanalyze.measure_selfdescr <- function (saveFile=FALSE, projectPhase="design", file="measure_self") {

# Plot constants 
selfdescrComp <- c("Software Developer", "Software Architect", "Other")

if(projectPhase == "design") {
    projectPhaseText <- "DESIGN"
    measures    <- designMeasures
    measuresLab <- designMeasuresLab
    nmDeveloper <- foss$CB_WHICH_MEASURES_DESIGN_PERS[foss$RB_DESCRIBE_YOURSELF %in% grep(selfdescr[1], foss$RB_DESCRIBE_YOURSELF, value=TRUE)]
    nmArchitect <- foss$CB_WHICH_MEASURES_DESIGN_PERS[foss$RB_DESCRIBE_YOURSELF %in% grep(selfdescr[2], foss$RB_DESCRIBE_YOURSELF, value=TRUE)]
    nmOther     <- foss$CB_WHICH_MEASURES_DESIGN_PERS[foss$RB_DESCRIBE_YOURSELF %in% grep(selfdescr[3], foss$RB_DESCRIBE_YOURSELF, value=TRUE)]
    fileName <- paste(file, "_design", sep = "")
} else if(projectPhase == "impl") {
    projectPhaseText <- "IMPLEMENTATION"
    measures    <- implMeasures
    measuresLab <- implMeasuresLab
    nmDeveloper <- foss$CB_WHICH_MEASURES_IMPL_PERS[foss$RB_DESCRIBE_YOURSELF %in% grep(selfdescr[1], foss$RB_DESCRIBE_YOURSELF, value=TRUE)]
    nmArchitect <- foss$CB_WHICH_MEASURES_IMPL_PERS[foss$RB_DESCRIBE_YOURSELF %in% grep(selfdescr[2], foss$RB_DESCRIBE_YOURSELF, value=TRUE)]
    nmOther     <- foss$CB_WHICH_MEASURES_IMPL_PERS[foss$RB_DESCRIBE_YOURSELF %in% grep(selfdescr[3], foss$RB_DESCRIBE_YOURSELF, value=TRUE)]
    fileName <- paste(file, "_impl", sep = "")
} else if(projectPhase == "test") {
    projectPhaseText <- "TESTING"
    measures <- testMeasures
    measuresLab <- testMeasureslab
    nmDeveloper <- foss$CB_WHICH_MEASURES_TEST_PERS[foss$RB_DESCRIBE_YOURSELF %in% grep(selfdescr[1], foss$RB_DESCRIBE_YOURSELF, value=TRUE)]
    nmArchitect <- foss$CB_WHICH_MEASURES_TEST_PERS[foss$RB_DESCRIBE_YOURSELF %in% grep(selfdescr[2], foss$RB_DESCRIBE_YOURSELF, value=TRUE)]
    nmOther     <- foss$CB_WHICH_MEASURES_TEST_PERS[foss$RB_DESCRIBE_YOURSELF %in% grep(selfdescr[3], foss$RB_DESCRIBE_YOURSELF, value=TRUE)]
    fileName <- paste(file, "_test", sep = "")
} else {
    projectPhaseText <- "MAINTENANCE"
    measures    <- mainMeasures
    measuresLab <- mainMeasuresLab
    nmDeveloper <- foss$CB_WHICH_MEASURES_MAINTNC_PERS[foss$RB_DESCRIBE_YOURSELF %in% grep(selfdescr[1], foss$RB_DESCRIBE_YOURSELF, value=TRUE)]
    nmArchitect <- foss$CB_WHICH_MEASURES_MAINTNC_PERS[foss$RB_DESCRIBE_YOURSELF %in% grep(selfdescr[2], foss$RB_DESCRIBE_YOURSELF, value=TRUE)]
    nmOther     <- foss$CB_WHICH_MEASURES_MAINTNC_PERS[foss$RB_DESCRIBE_YOURSELF %in% grep(selfdescr[3], foss$RB_DESCRIBE_YOURSELF, value=TRUE)]
    fileName <- paste(file, "_main", sep = "")
}

mainText <- paste("Measures for security assurance used during \nthe ", projectPhaseText, " (by self description).", sep = "")
ylabText=percenText;
xlabText="Self description";
legendLab <- measures

#Normirung Maximal Wert
normMax <- 100

nmDeveloperLen <- length(nmDeveloper)
nmArchitectLen <- length(nmArchitect)
nmOtherLen <- length(nmOther)

selfdescr_1 <- c(length(nmDeveloper[nmDeveloper %in% grep(measures[1], nmDeveloper, value=TRUE)])/nmDeveloperLen*normMax,
		length(nmDeveloper[nmDeveloper %in% grep(measures[2], nmDeveloper, value=TRUE)])/nmDeveloperLen*normMax,
		length(nmDeveloper[nmDeveloper %in% grep(measures[3], nmDeveloper, value=TRUE)])/nmDeveloperLen*normMax,
		length(nmDeveloper[nmDeveloper %in% grep(measures[4], nmDeveloper, value=TRUE)])/nmDeveloperLen*normMax,
		length(nmDeveloper[nmDeveloper %in% grep(measures[5], nmDeveloper, value=TRUE)])/nmDeveloperLen*normMax)

selfdescr_2 <- c(length(nmArchitect[nmArchitect %in% grep(measures[1], nmArchitect, value=TRUE)])/nmArchitectLen*normMax,
		length(nmArchitect[nmArchitect %in% grep(measures[2], nmArchitect, value=TRUE)])/nmArchitectLen*normMax,
		length(nmArchitect[nmArchitect %in% grep(measures[3], nmArchitect, value=TRUE)])/nmArchitectLen*normMax,
		length(nmArchitect[nmArchitect %in% grep(measures[4], nmArchitect, value=TRUE)])/nmArchitectLen*normMax,
		length(nmArchitect[nmArchitect %in% grep(measures[5], nmArchitect, value=TRUE)])/nmArchitectLen*normMax)

selfdescr_3 <- c(length(nmOther[nmOther %in% grep(measures[1], nmOther, value=TRUE)])/nmOtherLen*normMax,
		length(nmOther[nmOther %in% grep(measures[2], nmOther, value=TRUE)])/nmOtherLen*normMax,
		length(nmOther[nmOther %in% grep(measures[3], nmOther, value=TRUE)])/nmOtherLen*normMax,
		length(nmOther[nmOther %in% grep(measures[4], nmOther, value=TRUE)])/nmOtherLen*normMax,
		length(nmOther[nmOther %in% grep(measures[5], nmOther, value=TRUE)])/nmOtherLen*normMax)

colors <- c(hcl(h = c(120, 210), c = 20, l = 90, fixup = FALSE),"indianred","darkgray", "mintcream")

if(length(measures)>=6) {
    selfdescr_1 <- c(selfdescr_1, length(nmDeveloper[nmDeveloper %in% grep(measures[6], nmDeveloper, value=TRUE)])/nmDeveloperLen*normMax)
    selfdescr_2 <- c(selfdescr_2, length(nmArchitect[nmArchitect %in% grep(measures[6], nmArchitect, value=TRUE)])/nmArchitectLen*normMax)
    selfdescr_3 <- c(selfdescr_3, length(nmOther[nmOther %in% grep(measures[6], nmOther, value=TRUE)])/nmOtherLen*normMax)
    colors <- c(hcl(h = c(120, 210, 300), c = 20, l = 90, fixup = FALSE),"indianred","darkgray", "mintcream")
}
if(length(measures)>=7) {
    selfdescr_1 <- c(selfdescr_1, length(nmDeveloper[nmDeveloper %in% grep(measures[7], nmDeveloper, value=TRUE)])/nmDeveloperLen*normMax)
    selfdescr_2 <- c(selfdescr_2, length(nmArchitect[nmArchitect %in% grep(measures[7], nmArchitect, value=TRUE)])/nmArchitectLen*normMax)
    selfdescr_3 <- c(selfdescr_3, length(nmOther[nmOther %in% grep(measures[7], nmOther, value=TRUE)])/nmOtherLen*normMax)
    colors <- c(hcl(h = c(120, 210, 300, 400), c = 20, l = 90, fixup = FALSE),"indianred","darkgray", "mintcream")
}

# construct values vector
selfdescrVector <- c(selfdescr_1, selfdescr_2, selfdescr_3)

# add total number of members
lenVector <- c(nmDeveloperLen, nmArchitectLen, nmOtherLen)
selfdescrLab <- paste(selfdescrLab, "\n(", lenVector, " part.)", sep="")

# construct matrix to plot
plotMatrix <- matrix(selfdescrVector,
                nrow = length(measures), ncol = length(selfdescr), byrow = FALSE,
                dimnames = list(measures, selfdescrLab ))

# if "saveFile", than save as png image:
if (saveFile) png(paste(fileName, "png", sep="."), width=700, height=700)

#Plot parameters
par(mai = c(0.9, 0.9, 0.7, 0)) # margin size (bottom, left, top, right) specified in inches. 
par(mgp = c(3.5, 2, 0)) # margin line for (the axis title, axis labels, axis line)
par(cex.lab = 1.2) # label size 

#Draw Plot
barplot(plotMatrix, beside = TRUE, cex.names = 1.0, axis.lty = 0,
        col=colors,
        main=mainText, ylab=ylabText, xlab=xlabText,
        legend = rownames(plotMatrix),
        ylim = c(0, normMax))
grid(0, 10)

# if "saveFile", stop output-redirection to file
if (saveFile) dev.off()

}

#Function call with different parameters
isSaveFile <- TRUE
fossanalyze.measure_selfdescr(isSaveFile, "design")
fossanalyze.measure_selfdescr(isSaveFile, "impl")
fossanalyze.measure_selfdescr(isSaveFile, "test")
fossanalyze.measure_selfdescr(isSaveFile, "main")
